@@ -18,9 +18,8 @@ import lightgbm as lgb
 
 from sklearn.preprocessing import LabelEncoder
 from sklearn.model_selection import train_test_split
-from sklearn.metrics import mean_absolute_error
-from sklearn.metrics import classification_report, confusion_matrix
-from sklearn.metrics import f1_score
+from sklearn.metrics import mean_absolute_error, mean_squared_error, r2_score
+from sklearn.metrics import f1_score, classification_report, confusion_matrix
 
 
 from itertools import chain, combinations
@@ -78,7 +77,6 @@ class Snaplib:
         
     
 
-
     class Counter:
         def __init__(self):
             self.__counter = 0
@@ -87,11 +85,15 @@ class Snaplib:
             self.__counter += step
             return self.__counter
     
+
+
+
     
         
         
 
 
+
     def nan_info(self, df):
         '''
         Get pandas.DataFrame with information about missing data in columns
@@ -286,7 +288,7 @@ class Snaplib:
 
 
 
-    def encode_dataframe(self, df):
+    def encode_dataframe(self, df_0):
         '''
         encode a dataframe
 
@@ -294,13 +296,14 @@ class Snaplib:
         df = Snaplib().encode_dataframe(df)
         '''
 
-        if not isinstance(df, pd.core.frame.DataFrame):
+        if not isinstance(df_0, pd.core.frame.DataFrame):
             raise TypeError('The df must be a pandas.core.frame.DataFrame instance.')
 
         self.encoder_pool = {}
         self.decoder_pool = {}
         self.encoded_columns = []
 
+        df = df_0.copy()
         types = df.dtypes
         for col in df.columns:
             feature_type = types[col]
@@ -557,13 +560,14 @@ class Snaplib:
         global all_prediction_df
         global counter
         global cpu_num
-    
+        
         
         results=[]    
         all_prediction_df = pd.DataFrame()
         counter = self.Counter()
         research_best_score=dict()
         cpu_num = cpu_count()
+        num_combinations = 0
         
         
         def powerset(names_ls):
@@ -606,14 +610,23 @@ class Snaplib:
             print("%s %0.6f (std: +/- %0.2f)" % (metric_name, score, results_std))
             print('\n', results, '\n')
 
+
+            y_test = all_prediction_df.loc[:, 'Y_TEST']
+            y_pred = all_prediction_df.loc[:, 'Y_HAT_STACKED']
             if task == 'clsf':
-                print('\n', classification_report(all_prediction_df['Y_TEST'], all_prediction_df.loc[:, 'Y_HAT_STACKED']), '\n')
+                print('\n', classification_report(y_test, y_pred), '\n')
                 plt.figure(figsize=(3, 3))
-                cm = confusion_matrix(all_prediction_df.loc[:, 'Y_TEST'], all_prediction_df.loc[:, 'Y_HAT_STACKED'])
+                cm = confusion_matrix(y_test, y_pred)
                 heatmap(cm, annot=True, cmap="Blues", fmt='.0f',  cbar=False)
                 plt.show()
+            elif task == 'regr':
+                print('')
+                print('MAE    : %0.6f ' % mean_absolute_error(y_test, y_pred))
+                print('RMSE   : %0.6f ' % mean_squared_error(y_test, y_pred)**0.5)
+                print('R2     : %0.6f ' % r2_score(y_test, y_pred))
 
 
+            # SHOW PLOTS
             rng = np.random.default_rng()
             idx = set(rng.integers(len(all_prediction_df)-1, size=200))
 
@@ -648,14 +661,17 @@ class Snaplib:
 
                 all_combinations = list(powerset(alg_names))
                 num_combinations = len(all_combinations) -1
-                
+
+                list_metrics = [metric]*num_combinations
+                params = zip(all_combinations[1:], list_metrics)
+
                 results_pool = []
                 if task =='clsf':
                     with Pool(cpu_num) as p:
-                        results_pool.append(p.map(self.get_score_clsf, all_combinations[1:]))
+                        results_pool.append(p.map(self.get_score_clsf, params))
                 elif task=='regr':
                     with Pool(cpu_num) as p:
-                        results_pool.append(p.map(self.get_score_regr, all_combinations[1:]))
+                        results_pool.append(p.map(self.get_score_regr, params))
 
                 for line in results_pool[0]:
                     key, value = line[0], line[1]
@@ -664,6 +680,8 @@ class Snaplib:
                 print(f'\r{num_combinations} / {num_combinations} processed', end='', flush=True)
                 
 
+
+
                 best_list=[]
                 ascending = True if task=='clsf' else False
                 research_best_scoredict = sorted(research_best_score.items(), reverse=ascending)
@@ -704,7 +722,11 @@ class Snaplib:
                     ax[1].set_title('BEST COMPOSITION')
 
                     f.tight_layout()
+                    
                     plt.show()
+                    
+
+
 
                 elif task=='regr':
                     plt.figure(figsize=(20, 10))
@@ -719,30 +741,32 @@ class Snaplib:
                     plt.title('Compare all algorithms and the best')
                     plt.show()
                     
-                del num_combinations
-                del all_prediction_df
-                del counter
-                del cpu_num
+        del num_combinations
+        del all_prediction_df
+        del counter
+        del cpu_num
 
-            return score
+        return score
         
     @staticmethod
-    def get_score_clsf(alg_set):
+    def get_score_clsf(args):
         print(f'\r{counter(cpu_num)} / {num_combinations} processed', end='', flush=True)
-        algs_list = list(alg_set)
+        algs_list = list(args[0])
+        metric = args[1]
         y_hat_set = all_prediction_df.loc[:, algs_list].mode(axis=1)[0].astype('int64')
         score_set = metric(all_prediction_df.loc[:, ['Y_TEST']], y_hat_set)
         return score_set, algs_list
     
     @staticmethod
-    def get_score_regr(alg_set):
+    def get_score_regr(args):
         print(f'\r{counter(cpu_num)} / {num_combinations} processed', end='', flush=True)
-        algs_list = list(alg_set)
+        algs_list = list(args[0])
+        metric = args[1]
         y_hat_set = all_prediction_df.loc[:, algs_list].mean(axis=1)
         score_set = metric(all_prediction_df.loc[:, ['Y_TEST']], y_hat_set)
         return score_set, algs_list
-    
-    
+
+
 
 
 
@@ -926,7 +950,12 @@ class Snaplib:
         scores_df = pd.DataFrame(index = features, columns=['cv_score'])
         
         k_folds_dict_data = self.k_folds_split(df[features], target, cv)
-        score_with_all_features  = self.cross_val(algorithms, k_folds_dict_data, metric, task='clsf', cv=cv, verbose=0)
+        score_with_all_features  = self.cross_val(algorithms,
+                                                  k_folds_dict_data, 
+                                                  metric, 
+                                                  task='clsf', 
+                                                  cv=cv, 
+                                                  verbose=0)
         if verbose:
             print(f'{score_with_all_features}     General cv_score with all features')
         
@@ -934,7 +963,12 @@ class Snaplib:
         while score_with_all_features <= cv_score_of_bad_feature:
             scores_df = pd.DataFrame(index = features, columns=['cv_score'])
             k_folds_dict_data = self.k_folds_split(df[features], target, cv)
-            score_with_all_features  = self.cross_val(algorithms, k_folds_dict_data, metric, task='clsf', cv=cv, verbose=0)
+            score_with_all_features  = self.cross_val(algorithms, 
+                                                      k_folds_dict_data, 
+                                                      metric, 
+                                                      task='clsf', 
+                                                      cv=cv, 
+                                                      verbose=0)
             if verbose:
                 print('\n\n')
                 print(f'{len(features)} number of features')
@@ -946,7 +980,12 @@ class Snaplib:
                     fit_faetures = features[:]
                     fit_faetures.remove(without_feature)
                     k_folds_dict_data = self.k_folds_split(df[fit_faetures], target, cv)
-                    score  = self.cross_val(algorithms, k_folds_dict_data, metric, task='clsf', cv=cv, verbose=0)
+                    score  = self.cross_val(algorithms, 
+                                            k_folds_dict_data, 
+                                            metric, 
+                                            task='clsf', 
+                                            cv=cv, 
+                                            verbose=0)
                     scores_df.loc[without_feature] = score
                     if verbose:
                         print("{:1.8f}   {:20}  ".format(score, without_feature))
@@ -959,9 +998,9 @@ class Snaplib:
                 features.remove(bad_feature)
                 droped_features.append(bad_feature)
                 if verbose:
-                    print('--------------------------------------------')
+                    print('='*50,'\n')
                     print(f'    APPEND TO DROP    {bad_feature}')
-                    print('--------------------------------------------')
+                    print('='*50,'\n')
             else:
                 if verbose:
                     print('\n\n')
@@ -1166,7 +1205,7 @@ class Snaplib:
             df = df.sort_values(by=important_functions, ascending=True)
             df = df.reset_index(drop=True)
             if research_iter:
-                print('\n-----------------------------------')
+                print('='*50,'\n')
                 print(f'\nThe Table has been ordered and sorted by columns:\n\n{important_functions}')
             return df
 
@@ -1232,7 +1271,7 @@ class Snaplib:
         df_count = df_count.sort_values(by=columns, ascending=True)
 
         if research_iter:
-            print('\n-----------------------------------')
+            print('='*50,'\n')
             print(df_count)
         
         ordered_predictors = list(df_count.index)
@@ -1251,7 +1290,7 @@ class Snaplib:
         train_test_split_ordered(get_X(df, predictors), get_y(df, target_feature), random_state, test_size=test_size)
 
         if research_iter:
-            print('\n-----------------------------------')
+            print('='*50,'\n')
             if  len(train_y.value_counts()) > CLASSIFIER_FOR_UNIQUE_VALUES_LESS_THAN:
                 print(f'The final result MAE of the custom split:\n\
                 \nMAE:  {regression_score(train_X, test_X, train_y, test_y)}')
@@ -1259,7 +1298,7 @@ class Snaplib:
                 print(f'The final result F1_SCORE(average="macro") of the custom split:\n\nF1_SCORE: \
                 {classification_accuracy(train_X, test_X, train_y, test_y)}')
         if research_iter:
-            print('\n-----------------------------------')
+            print('='*50,'\n')
             finish_time = datetime.datetime.now()
             requared_time = finish_time - init_time
             print(f'Required time:  {str(requared_time)}\n')
@@ -1509,7 +1548,7 @@ class Snaplib:
                 print(f'Imputed Values: {count_miss_values}')
                 print(f'Required time:  {str(requared)}\n')
 
-        # return dataframe state to their initial states (decode, index, types)
+        # return dataframe states to their initial states (decode, index, types)
         df = self.decode_dataframe(df)
 
         for col in df.columns:
